@@ -3,7 +3,8 @@
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
     2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
-    2017, 2018, 2019, 2020 Massachusetts Institute of Technology
+    2017, 2018, 2019, 2020, 2021, 2022 Massachusetts Institute of
+    Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -33,7 +34,7 @@ USA.
 		'(runtime number)
 		'(runtime number-parser))
 
-(define last-copyright-year 2020 )
+(define last-copyright-year 2022 )
 
 (define-deferred copyright-years
   (let ((then 1986))
@@ -42,7 +43,7 @@ USA.
 
 (add-boot-init!
  (lambda ()
-   (add-subsystem-identification! "Release" '(11 2))
+   (add-subsystem-identification! "Release" '(12 0 90))
    (run-now-and-after-restore!
     (lambda ()
       (add-subsystem-identification! "Microcode" (get-microcode-version))))
